﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using Syncfusion.Drawing;
using Syncfusion.Pdf;
using Syncfusion.Pdf.Graphics;

namespace PDFFormTesting
{
	public class DocumentAssembler
	{
		private string[] _imagePaths;
		private string _outputFolder;
		private string _outputFileName;

		public DocumentAssembler(string[] imagePaths, string outputFolder, string outputFileName)
		{
			_imagePaths = imagePaths;
			_outputFolder = outputFolder;
			_outputFileName = outputFileName;
		}

		public void AssembleDocumentAsPortableDocumentFormat()
		{
			try
			{
				if (!Directory.Exists(_outputFolder))
					Directory.CreateDirectory(_outputFolder);
			}
			catch (IOException ex)
			{
				Debug.WriteLine("Failed to create directory with exception: " + ex.ToString());
			}

			var document = new PdfDocument();
			document.PageSettings.Margins.All = 0.0f;

			foreach (var path in _imagePaths)
			{
				var page = document.Pages.Add();
				var pageGraphics = page.Graphics;
				using (var fileStream = new FileStream(path, FileMode.Open))
				{
					var pdfImage = PdfImage.FromStream(fileStream);
					pageGraphics.DrawImage(pdfImage, CenteredRectangleThatFitsInPage(page, pdfImage));
				}
			}

			using (var documentMemoryStream = new MemoryStream())
			{
				document.Save(documentMemoryStream);
				document.Close(true);

				SaveDocument(documentMemoryStream);
				documentMemoryStream.Flush();
			}
		}

		private static float CalculateScaleToFitToPage (PdfPage page, PdfImage pdfImage)
		{
			var widthScale = page.Graphics.ClientSize.Width / pdfImage.Width;
			var heightScale = page.Graphics.ClientSize.Height / pdfImage.Height;
			var scale = Math.Min (widthScale, heightScale);

			return scale;
		}

		private static RectangleF CenteredRectangleThatFitsInPage (PdfPage page, PdfImage pdfImage)
		{
			float scale = CalculateScaleToFitToPage (page, pdfImage);

			var scaledWidth = pdfImage.Width * scale;
			var scaledHeight = pdfImage.Height * scale;

			var horizontalOffset = (page.Graphics.ClientSize.Width - scaledWidth) / 2.0f;
			var verticalOffset = (page.Graphics.ClientSize.Height - scaledHeight) / 2.0f;

			return new RectangleF (horizontalOffset, verticalOffset, scaledWidth, scaledHeight);
		}

		private void SaveDocument(MemoryStream documentMemoryStream)
		{
			var filePath = Path.Combine(_outputFolder, _outputFileName);
			try
			{
				using (var fileStream = File.Open(filePath, FileMode.Create))
				{
					documentMemoryStream.Position = 0;
					documentMemoryStream.CopyTo(fileStream);
				}
			}
			catch (IOException ex)
			{
				Debug.WriteLine("Failed to save pdf with exception: " + ex.ToString());
			}
		}
	}
}
